/*-
 * Copyright (c) 2000, 2002, 2006 Roman V. Palagin
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#include <sys/cdefs.h>
__RCSID("$Id: kernel.c,v 1.5 2006/06/22 11:56:38 romanp Exp $");

#include <sys/param.h>
#include <sys/sysctl.h>

#include <limits.h>
#include <fcntl.h>
#include <kvm.h>
#include <nlist.h>
#include <stdio.h>
#include <stdlib.h>
#include <err.h>

#include "kernel.h"

static kvm_t 	*kd;		          /* kernel descriptor */
static 	char	errbuf[_POSIX2_LINE_MAX]; /* buffer for kvm(3) error reporting */

int
k_open(void) 
{
	kd = kvm_openfiles(NULL, NULL, NULL, O_RDONLY, errbuf);
	if (kd == NULL)
		errx(1, "kvm_openfiles: %s\n", errbuf);
	return 0;
}	

int 
k_read(u_long addr, void *buf, size_t nbytes)
{
	ssize_t rc = kvm_read(kd, addr, buf, nbytes);

	if (rc != (ssize_t)nbytes) { /* error occured */
		warnx("kvm_read: %s\n", kvm_geterr(kd));
		return -1;
	} else return 0;
}

int
k_close(void)
{
	int rc = kvm_close(kd);
    
	if (rc == -1)
		warnx("kvm_close: %s\n", kvm_geterr(kd));
	return rc;
}

struct kinfo_proc *
k_getallprocs(int *nprocs)
{
	struct kinfo_proc *rc = kvm_getprocs(kd, KERN_PROC_ALL, 0, nprocs);
    
	if (rc == NULL) 
		errx(1, "kvm_getprocs: %s\n", kvm_geterr(kd));
	return rc;
}

struct kinfo_proc *
k_getproc(pid_t pid)
{
	int		cnt;
	struct 	kinfo_proc *rc;

	rc = kvm_getprocs(kd, KERN_PROC_PID, pid, &cnt);
    
	if (rc == NULL) 
		errx(1, "kvm_getprocs: %s\n", kvm_geterr(kd));
	return rc;
}

int 
k_nlist(struct nlist *nl)
{
	int rc = kvm_nlist(kd, nl);
    
	if (rc)
		errx(1, "kvm_nlist: %s\n", kvm_geterr(kd));
	return 0;
}

